﻿# 計算書Word整形プログラム
#
# 初版作成者・作成日：合同会社ＧＤウイング 2020/01/18
# 
# 必ずチェックのうえ、ご使用ください。

import os
import sys
import tkinter.filedialog
import tkinter.messagebox
import win32com.client

#####################
### 1.初期設定部  ###
#####################

# (1) ファイル選択のパスを、プログラムと同じフォルダにセットする。
currentdir = os.path.dirname(os.path.abspath(__file__))

# (2) プログラム名（有効なExcelデータであるかを識別するための文字列。）
program_name = "GD計算書Word整形_V1"

#######################
### 2.入力部(Excel) ###
#######################

# 関数定義 ファイルオープン
def OpenFile():
    # Tkの小さいウィンドウが出る場合は以下の２行を追加すると非表示になる。
    root = tkinter.Tk()
    root.withdraw()

    #タイトル、選択可能ファイル拡張子、初期ディレクトリ
    filetype = [("入力データ(*.xlsx)","*.xlsx")]

    #ファイル選択ダイアログ表示
    xl_filename = tkinter.filedialog.askopenfilename(title = "入力データを選択してください。", filetypes = filetype, initialdir = currentdir)

    # キャンセルボタンを選択した場合は、メッセージボックスを表示してプログラムを終了する。ファイルを選択した場合は、inputfileを返す。
    if xl_filename == "":
        tkinter.messagebox.showinfo(program_name, "データは選択されませんでした。\r\nプログラムを終了します。")
        sys.exit(0)
    else:
        return xl_filename

# クラス定義 Excelの表をテーブルデータとして読み込む。
class BuildTable(object):
    # １レコード分をExcelから取得し、まとめてtableに追加する。
    data = []      # 全データの辞書格納用配列
    def __init__(self, xl_sheet, xl_start, xl_end):
        record_num = 1  # レコード番号

        while xl_start.Column + record_num <= xl_end.Column:
            record = [{}]           # 1レコード作業用辞書
            xl_item = xl_start.Row  # 項目行のループカウンタ
            if xl_sheet.cells(xl_item, xl_start.Column + record_num).text == "":
                record_num += 1
                continue
            else:
                while xl_item <= xl_end.Row:
                    record[0][xl_sheet.cells(xl_item, xl_start.Column).text] = xl_sheet.cells(xl_item, xl_start.Column + record_num).text
                    xl_item += 1
            self.data.extend(record)
            record_num += 1

    # 受け取った文字列の変数化および、計算結果を書式整形しテーブルに追加する関数
    def AddTable(self, record_num, addname, digit, addanser):
        exec(addname + "=" + str(addanser), globals())
        self.data[record_num][addname] = f'{addanser:,.{digit}f}'

## 2-1.Excel起動 ##
inputfile = OpenFile()
# Excelデータのパス名を取得(後でWordデータの出力先として使う)
filepath = os.path.dirname(inputfile)

excel = win32com.client.Dispatch("Excel.Application")
excel.Visible = True

## 2-2.対象のExcelブック取得 ##
try:
    book = excel.Workbooks.Open(inputfile)
    sheet = book.Worksheets("入力データ")
except:
    # Excel終了
    excel.Quit()

    # メッセージボックス表示およびプログラム終了
    tkinter.messagebox.showwarning(program_name, "読み込まれたファイルは本プログラム用のデータではありません。\r\nプログラムを終了します。")
    sys.exit(0)

# "リリース"：置換したセルは着色されない。"デバッグ"：置換したセルは着色される。
debug_flag = sheet.Cells(1,15).Value

## 2-3.表題部の取得 ##
if sheet.Cells(1,4).Value != program_name:
    # Excel終了
    excel.Quit()
    # メッセージボックス表示およびプログラム終了
    tkinter.messagebox.showwarning(program_name, "読み込まれたファイルは`本プログラム用のデータではありません。\r\nプログラムを終了します。")
    sys.exit(0)

formatfile = sheet.Cells(2,4).Value + ".docx" # ワード書式ファイル名
text1 = sheet.Cells(4,4).Value
text2 = sheet.Cells(5,4).Value

## 2-4.計算ケースごとの入力データ取得 ##
## (1) 入力データ範囲の設定(表の左上と右下を指定する) #

start_row_num = sheet.cells(5,14).Value
start_col_num = sheet.cells(5,15).Value
end_row_num = sheet.cells(6,14).Value
end_col_num = sheet.cells(6,15).Value

start = sheet.cells(start_row_num,start_col_num) # 左上セル
end = sheet.cells(end_row_num,end_col_num)  # 右下セル

## (2) 各入力データ取得
# BuildTableクラスによりデータを取得する。
table = BuildTable(sheet, start, end)

## 2-5.Excel終了 ##
excel.Quit()

######################
### 3.出力部(Word) ###
######################

# 関数定義
def replacetext(rt_before, rt_after):
    if debug_flag == "リリース":
        while word.Selection.Find.Execute("#" + rt_before + "#") == True:
            # 検索箇所のハイライトを無色にする。
            word.Selection.range.HighlightColorIndex = 0

        # 検索漏れを防ぐために文書の先頭にカーソルを戻す。
        word.Selection.GoTo(1, 1, 1, 1)

    elif debug_flag == "デバッグ" and rt_after == None:
        # デバッグ版を出力する場合、NULL値の変数に"※NULL※"と表示させる。
        rt_after = "※NULL※"

    if word.Selection.Find.Execute("#" + rt_before + "#", True, False, False, False, False, True, 1, False, rt_after, 2) != True:
        return 1    # None以外を返して、エラーフラグを出す。

def delete_area(delete_text):
    # 削除対象範囲を削除する。
    word.Selection.GoTo(1, 1, 1, 1)
    while word.Selection.Find.Execute("$" + delete_text + "_START$") == True:
        delete_start_pos = word.Selection.range.start
        word.Selection.GoTo(1, 1, 1, 1)
        word.Selection.Find.Execute("$" + delete_text + "_END$")
        delete_end_pos = word.Selection.range.end
        # 範囲の最後に1を足して、改行も含めて選択する。
        word.Selection.GoTo(1, 1, 1, 1)
        word.Selection.SetRange(delete_start_pos,delete_end_pos + 1)
        word.Selection.Delete()
        word.Selection.GoTo(1, 1, 1, 1)

def leave_area(leave_text):
    word.Selection.GoTo(1, 1, 1, 1)
    # 削除対象外範囲は、$で囲まれた記号のみ削除する。
    while word.Selection.Find.Execute("$" + leave_text + "_START$") == True:
        word.Selection.SetRange(word.Selection.range.start, word.Selection.range.end + 1)
        word.Selection.Delete()
        word.Selection.GoTo(1, 1, 1, 1)
        word.Selection.Find.Execute("$" + leave_text + "_END$")
        # 範囲の最後に1を足して、改行も含めて選択する。
        word.Selection.SetRange(word.Selection.range.start, word.Selection.range.end + 1)
        word.Selection.Delete()
        word.Selection.GoTo(1, 1, 1, 1)

## 3-1.Word起動 ##
word = win32com.client.Dispatch("Word.Application")
word.Visible = True
#word.DisplayAlerts = True

## 3-2.$table内の各データを複数ファイルに出力
data_num = 0
while data_num <= len(table.data) - 1:
    # 様式ファイル(Wordドキュメント)をオープン
    try:
        outputfile = word.documents.Open(filepath + "/" + formatfile)
    except:
        # Word終了
        word.Quit()

        # メッセージボックス表示およびプログラム終了
        tkinter.messagebox.showwarning(program_name, "書式ファイルが見つかりません。\r\nプログラムを終了します。")
        sys.exit(0)

    # Wordの機能を利用して文字列置換する(表題部)
    replacetext("text1", text1)
    replacetext("text2", text2)
    # 指定範囲を抽出する
    # 範囲1
    delete_area(table.data[data_num]["delete_cat_1"])
    leave_area(table.data[data_num]["leave_cat_1"])
    # 範囲2
    delete_area(table.data[data_num]["delete_cat_2"])
    leave_area(table.data[data_num]["leave_cat_2"])
    # 範囲3
    delete_area(table.data[data_num]["delete_cat_3"])
    leave_area(table.data[data_num]["leave_cat_3"])

    for key in table.data[data_num]:
        target_str = table.data[data_num][key].replace(",","")

    for key in table.data[data_num]:
        des_num = table.data[data_num][key]

        # Wordの機能を利用して文字列置換する(設計数値)
        replace_flag = replacetext(key, des_num)

        if replace_flag != None:
            if key == "leave_cat_1" or key == "leave_cat_2" or key == "leave_cat_3":
                continue
            elif key == "delete_cat_1" or key == "delete_cat_2" or key == "delete_cat_3":
                continue
            else:
                tkinter.messagebox.showinfo(program_name, \
                    "以下の記号と変数は出力に使われていません。続行します。\r\n変数名：「" + key + "」" + "　値：「" + des_num + "」")

    # Excelデータにおいて識別名"CAUTION"を使用した場合は、ファイル名に識別の内容を付加する
    if table.data[data_num]["CAUTION"] != "":
        caution_fname_add = "[" + table.data[data_num]["CAUTION"] + "]"
    else:
        caution_fname_add = ""

    # 出力ファイルとしてWordドキュメントを別名保存
    report_file = text1 + text2 + table.data[data_num]["text3"] + caution_fname_add +".docx"
    outputfile.SaveAs(filepath + "/" + report_file)
    outputfile.Application.ActiveDocument.Close()

    data_num += 1

## 3-3.Word終了 ##
word.Quit()